package cn.efunbox.manage.base.service.impl;

import cn.efunbox.manage.base.entity.Department;
import cn.efunbox.manage.base.entity.Product;
import cn.efunbox.manage.base.entity.Project;
import cn.efunbox.manage.base.enums.BaseStatusEnum;
import cn.efunbox.manage.base.repository.DepartmentRepository;
import cn.efunbox.manage.base.repository.ProductRepository;
import cn.efunbox.manage.base.repository.ProjectRepository;
import cn.efunbox.manage.base.service.ProductService;
import cn.efunbox.manage.common.result.ApiCode;
import cn.efunbox.manage.common.result.ApiResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * ProductServiceImpl
 * Created by wangys on 2019/03/07
 */
@Service
public class ProductServiceImpl implements ProductService {

    @Autowired
    ProductRepository productRepository;

    @Autowired
    ProjectRepository projectRepository;

    @Autowired
    DepartmentRepository departmentRepository;

    @Override
    public ApiResult list(Product product) {
        List<Product> products = productRepository.find(product);
        if(Objects.nonNull(products)){
            return ApiResult.ok(products);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult save(Product product) {
        if(product ==null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        product = productRepository.save(product);
        if(Objects.nonNull(product)){
            return ApiResult.ok(product);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult update(Product product) {
        if(product ==null){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        product = productRepository.update(product);
        if(Objects.nonNull(product)){
            return ApiResult.ok(product);
        }
        return ApiResult.error(ApiCode.UNKNOWN_ERROR);
    }

    @Override
    public ApiResult getProductById(Long productId) {
        if(Objects.isNull(productId)){
            return ApiResult.error(ApiCode.PARAMETER_ERROR);
        }
        Product product = productRepository.find(productId);
        if(product ==null){
            return ApiResult.error(ApiCode.NOT_FOUND);
        }
        return ApiResult.ok(product);
    }

    @Override
    public ApiResult getSelect(){
        return ApiResult.ok(productRepository.findByStatusOrderBySortDesc(BaseStatusEnum.NORMAL));
    }
}
